﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TechnoAP.usbmca.device
{
    using FTD2XX_NET;
    using TechnoAP.usbmca;

    /// <summary>
    /// Summary for device
    /// </summary>
    public class device : FTD2XX_NET.FTDI
    {
        public FTDI.FT_STATUS mLastStaus;
        public string mLastCommand;
        public uint mLastWriteCommandValue;
        public uint mLastReadCommandValue;
        public int mnErrorReadList = 0;

        private uint mInputDummy = 0 ;
        private uint mOutputDummy = 0;

        public status mStatus = new status();
        public uint[,] maHistogram = new uint[constant.Max.CH, constant.Size.HistogramMaxCh];

        /// <summary>
        /// static constractor
        /// </summary>
        static device()
        {
            // Adding environment variables(PATH), Folder with driver DLL(ftd2xx.dll)
            string strOldPath = System.Environment.GetEnvironmentVariable("PATH", EnvironmentVariableTarget.Process);
            string strSystemPath = System.Environment.GetFolderPath(Environment.SpecialFolder.System);
            string strDriverPath = strOldPath + ";" + strSystemPath + "\\TechnoAP";
            System.Environment.SetEnvironmentVariable("PATH", strDriverPath, EnvironmentVariableTarget.Process);
        }

        /// <summary>
        /// Write command message
        /// </summary>
        public bool WriteCommand(string command, int channel, uint value)
        {
            FTDI.FT_STATUS statResult;
            bool blnResult = false;

            // Generation of command telegram
            byte[] abCommand = common.command.createTelegram(command, channel, value);

            mLastWriteCommandValue = value;

            // Write command
            uint nBytesWritten = 0;
            statResult = this.Write(abCommand, abCommand.Length, ref nBytesWritten);

            mLastStaus = statResult;

            if (statResult == FT_STATUS.FT_OK && nBytesWritten == abCommand.Length)
            {
                blnResult = true;
            }

            return blnResult;

        }

        /// <summary>
        /// Read command response message
        /// </summary>
        public bool ReadCommand(ref string command, ref uint value)
        {
            FTDI.FT_STATUS statResult;
            bool blnResult = false;

            byte[] abCommand = new byte[8];

            uint nBytesRead = 0;
            statResult = this.Read(abCommand, (uint)(abCommand.Length), ref nBytesRead);

            mLastStaus = statResult;

            if (statResult == FT_STATUS.FT_OK && nBytesRead == abCommand.Length)
            {
                command = common.command.restoreCommand(abCommand, ref value);
                mLastReadCommandValue = value;
                blnResult = true;
            }

            return blnResult;

        }

        /// <summary>
        /// Command execution
        /// </summary>
        public bool OrderCommand(string command, int channel, uint input, ref uint output)
        {
            bool blnResult = false;
            bool bln1st, bln2nd;

            // Write request command
            bln1st = WriteCommand(command, channel, input);

            if (bln1st == true)
            {
                // Read response command
                string dummy = string.Empty;
                bln2nd = ReadCommand(ref dummy, ref output);
                blnResult = bln2nd;
            }

            return blnResult;

        }

        /// <summary>
        /// Start measurement
        /// </summary>
        public bool StartMeasurement()
        {
            return OrderCommand(constant.Command.AQSW, (int)constant.CH_TYPE.Common, 1, ref mOutputDummy);
        }

        /// <summary>
        /// Stop measurement
        /// </summary>
        public bool StopMeasurement()
        {
            return OrderCommand(constant.Command.AQEW, (int)constant.CH_TYPE.Common, 1, ref mOutputDummy);
        }

        /// <summary>
        /// Clear measurement data
        /// </summary>
        public bool ClearMeasurementData()
        {
            return OrderCommand(constant.Command.CLRW, (int)constant.CH_TYPE.Common, mInputDummy, ref mOutputDummy);
        }

        /// <summary>
        /// Setting measurement mode(historam or list)
        /// </summary>
        public bool SetMeasurementDataMode(constant.DataMode value)
        {
            return OrderCommand(constant.Command.MODW, (int)constant.CH_TYPE.Common, (uint)value, ref mOutputDummy);
        }

        /// <summary>
        /// Setting measurement time mode(real or live)
        /// </summary>
        public bool SetMeasurementTimeMode(constant.TimeMode value)
        {
            return OrderCommand(constant.Command.MMDW, (int)constant.CH_TYPE.Common, (uint)value, ref mOutputDummy);
        }

        /// <summary>
        /// Setting measurement time
        /// </summary>
        public bool SetMeasurementTime(long value)
        {
            bool blnResult = false;

            uint value0 = common.binary.getMsb32bit(value) & 0x0fff;
            bool bln0th = OrderCommand(constant.Command.MT0W, (int)constant.CH_TYPE.Common, value0, ref mOutputDummy);

            if (bln0th == true)
            {
                uint value1 = common.binary.getLsb32bit(value);
                bool bln1st = OrderCommand(constant.Command.MT1W, (int)constant.CH_TYPE.Common, value1, ref mOutputDummy);
                blnResult = bln1st;
            }

            return blnResult;
        }

        /// <summary>
        /// Setting of measurement time (in seconds)
        /// </summary>
        public bool SetMeasurementTimeSeconds(long value)
        {
            long lngCount = common.convert.SecondsToCount(value);
            return SetMeasurementTime(lngCount);
        }

        /// <summary>
        /// Setting of measurement time (in milliseconds)
        /// </summary>
        public bool SetMeasurementTimeMilliseconds(long value)
        {
            long lngCount = common.convert.MillisecondsToCount(value);
            return SetMeasurementTime(lngCount);
        }

        /// <summary>
        /// Reading measurement status
        /// </summary>
        public status ReadStatus()
        {
            status statResult = null;

            // Measurement state request
            bool blnReuslt = WriteCommand(constant.Command.STUW, (int)constant.CH_TYPE.Common, 0);

            if (blnReuslt == true)
            {
                // Reading measurement status(response data)
                byte[] abStatus = new byte[94];
                uint uBytesRead = 0;
                FT_STATUS ftResult = Read(abStatus, (uint)abStatus.Length, ref uBytesRead);

                if (ftResult == FT_STATUS.FT_OK && abStatus.Length == (int)(uBytesRead))
                {
                    // Organization of measurement state data
                    mStatus.analyzeData(abStatus);
                    statResult = mStatus;
                }
            }

            return statResult;

        }

        /// <summary>
        /// Read list data
        /// </summary>
        public byte[] ReadListData(ref int nReadBytes)
        {
            bool bln1st, bln2nd;
            byte[] abListData = null;

            nReadBytes = 0;

            // Request list data
            bln1st = WriteCommand(constant.Command.LISR, (int)constant.CH_TYPE.Common, 0);

            if (bln1st == true)
            {
                // Read response command
                uint uOutput = 0;
                string strCommand = string.Empty;
                bln2nd = ReadCommand(ref strCommand, ref uOutput);

                if (bln2nd == true)
                {
                    if (strCommand.CompareTo(constant.Command.LISA) == 0)
                    {
                        int nBlock = (int)(uOutput) & 0x0000ffff;
                        int nBytes = (int)(nBlock * (uint)constant.Size.ListDataBlock);

                        if(nBytes > 0)
                        {
                            // There is list data -> Reading data
                            uint nReadBytesWork = 0;
                            abListData = new byte[nBytes];
                            Read(abListData, (uint)(nBytes), ref nReadBytesWork);
                            nReadBytes = (int)(nReadBytesWork);
                        }
                    }
                    else
                    {
                        // Invalid response command
                        mnErrorReadList = -3;
                    }
                }
                else
                {
                    // Failed to read response command
                    mnErrorReadList = -2;
                }
            }
            else
            {
                // Command write failure
                mnErrorReadList = -1;
            }

            return abListData;

        }

        /// <summary>
        /// Reading histogram data
        /// </summary>
        /// <param name="channel">target CH</param>
        /// <param name="maxch">Max ch count(16384, 8192, 4096~)</param>
        /// <returns>command text</returns>
        public uint[] ReadHistogram(int channel, int maxch)
        {
            uint[] auHistogram = new uint[constant.Size.HistogramMaxCh];
            bool bln1st, bln2nd;
            uint uInput, uOutput;

            Array.Clear(auHistogram, 0, auHistogram.Length);

            // Target CH switching
            uInput = (uint)(channel - 1);
            uOutput = 0;
            bln1st = OrderCommand(constant.Command.HCHW, (int)constant.CH_TYPE.Common, uInput, ref uOutput);

            if (bln1st == true)
            {
                uint uRxBytes = 0;
                byte[] abRxData = new byte[constant.Size.HistogramOnceByte];
                Array.Clear(abRxData, 0, abRxData.Length);

                int nRxCount = maxch / constant.Size.HistogramOnceCh;

                for (int i = 0; i < nRxCount; i++)
                {
                    // Request the histogram data of the block
                    string command = string.Format(constant.Command.HIXX, i);

                    bln2nd = WriteCommand(command, (int)constant.CH_TYPE.Common, 0);

                    if (bln2nd == false)
                    {
                        auHistogram = null;
                        break;
                    }

                    // Read out the histogram data of this block
                    uint uBytesRead = 0;
                    FTDI.FT_STATUS statResult = this.Read(abRxData, (uint)abRxData.Length, ref uBytesRead);

                    if (statResult != FT_STATUS.FT_OK || uBytesRead != abRxData.Length)
                    {
                        mLastStaus = statResult;
                        auHistogram = null;
                        break;
                    }

                    Buffer.BlockCopy(abRxData, 0, auHistogram, (int)uRxBytes, (int)uBytesRead);
                    uRxBytes += uBytesRead;
                }

                // Convert to host byte order
                for (int i = 0; i < auHistogram.Length; i++)
                {
                    auHistogram[i] = (uint)(System.Net.IPAddress.HostToNetworkOrder((int)auHistogram[i]));
                    maHistogram[(channel - 1), i] = auHistogram[i];
                }
            }

            return auHistogram;

        }
    }
}
